import { entityIsArchetype } from "./util/entityIsArchetype";
import { ListenerRegistry } from "./util/ListenerRegistry";
export class Archetype {
    constructor(query) {
        this.query = query;
        /** A list of entities belonging to this archetype. */
        this.entities = new Array();
        /** Listeners on this event are invoked when an entity is added to this archetype's index. */
        this.onEntityAdded = new ListenerRegistry();
        /** Listeners on this event are invoked when an entity is removed from this archetype's index. */
        this.onEntityRemoved = new ListenerRegistry();
    }
    indexEntity(entity) {
        const isArchetype = entityIsArchetype(entity, this.query);
        const pos = this.entities.indexOf(entity, 0);
        if (isArchetype && pos < 0) {
            this.entities.push(entity);
            this.onEntityAdded.invoke(entity);
        }
        else if (!isArchetype && pos >= 0) {
            this.entities.splice(pos, 1);
            this.onEntityRemoved.invoke(entity);
        }
    }
    removeEntity(entity) {
        const pos = this.entities.indexOf(entity, 0);
        if (pos >= 0) {
            this.entities.splice(pos, 1);
            this.onEntityRemoved.invoke(entity);
        }
    }
}
